from setuptools import setup

setup(name = 'wrsi',
      version = '0.1',
      description = 'Water requirement satisfaction index (wrsi) for crop calculator',
      packages = ['wrsi'],
      author = 'Farafehizoro Ramanjatonirina',
      zip_safe = False)

